@rem = '--*-Perl-*--
@echo off
if "%OS%" == "Windows_NT" goto WinNT
perl -x -S "%0" %1 %2 %3 %4 %5 %6 %7 %8 %9
goto endofperl
:WinNT
perl -x -S "%0" %*
if NOT "%COMSPEC%" == "%SystemRoot%\system32\cmd.exe" goto endofperl
if %errorlevel% == 9009 echo You do not have Perl in your PATH.
goto endofperl
@rem ';
#!perl
#line 14

############################################
#                                          #
#  The script "wav2mp3bat.bat" is          #
#                                          #
#  Copyright (C) 2005 by Steffen Beyer.    #
#  All rights reserved.                    #
#                                          #
#                                          #
#  The script "wav2mp3bat.bat" is          #
#                                          #
#  free software; you can redistribute it  #
#  and/or modify it under the same terms   #
#  as Perl itself.                         #
#                                          #
############################################

use strict;

my($Self,$Src,$Tgt,@Task,$Item);

$Self = $0; $Self =~ s!^.*[\/\\]!!; $Self =~ s!\.[^\.\/\\]*$!!;

unless (@ARGV and (@ARGV == ((@ARGV >> 1) << 1)))
{
    die "Usage: $Self [ <SourceDir> <TargetDir> ]+\n";
}

while (@ARGV)
{
    $Src = shift;
    $Tgt = shift;
    unless (opendir(SRC, $Src))
    {
        warn "$Self: can't open directory <$Src>: $!\n";
        next;
    }
    @Task = map { [ "$Src/$_", "$Tgt/$_" ] }
            sort
            grep { !/^\.\.?$/ and (-d "$Src/$_") }
            readdir(SRC);
    unless (closedir(SRC))
    {
        warn "$Self: can't close directory <$Src>: $!\n";
    }
    foreach $Item (@Task) { Process($Item); }
    print "\n\nDONE.\a\a\a\a\a\n\n";
}

sub Process
{
    my($Ref) = shift;
    my(@List,@Meta,$WAV,$MP3,$Err,$File,$Artist,$Album,$Year,$Genre,$Count,$Title,$Track,$Filename,$Outname,$Newname,$CMD);

    ($WAV,$MP3) = @$Ref;
    unless (opendir(WAV, $WAV))
    {
        warn "$Self: can't open directory <$WAV>: $!\n";
        return;
    }
    @List = grep { !/^\.\.?$/ }
            readdir(WAV);
    unless (closedir(WAV))
    {
        warn "$Self: can't close directory <$WAV>: $!\n";
    }
    @Meta = grep { /\.txt$/i }
            @List;
    @List = map  { $_->[0] }
            sort { $a->[1] <=> $b->[1] || lc($a->[0]) cmp lc($b->[0]) || $a->[0] cmp $b->[0] }
            map  { if (/^(\d+)/) { [$_,$1] } elsif (/^.*?\D(\d+)\.wav$/i) { [$_,$1] } else { [$_,0] } }
            grep { (-f "$WAV/$_") and /^(?:\d+.*|.*?\D\d+)\.wav$/i }
            @List;
    if (@Meta == 0)
    {
        warn "$Self: no manifest (TXT) file found in directory <$WAV>!\n";
        return;
    }
    if (@Meta > 1)
    {
        warn "$Self: more than one possible manifest (TXT) file found in directory <$WAV>!\n";
        return;
    }
    if (@List == 0)
    {
        warn "$Self: no input (WAV> files found in directory <$WAV>!\n";
        return;
    }
    else
    {
        if ($Err = MakeDir($MP3))
        {
            warn "$Self: $Err\n";
            return;
        }
    }
    $File = $WAV . '/' . shift(@Meta);

    unless (open(FILE, "<$File"))
    {
        warn "$Self: can't open manifest file <$File>: $!\n";
        return;
    }

    unless (defined ($Artist = <FILE>))
    {
        close(FILE);
        warn "$Self: can't find field <Artist> in manifest file <$File>!\n";
        return;
    }
    unless (defined ($Album = <FILE>))
    {
        close(FILE);
        warn "$Self: can't find field <Album> in manifest file <$File>!\n";
        return;
    }
    unless (defined ($Year = <FILE>))
    {
        close(FILE);
        warn "$Self: can't find field <Year> in manifest file <$File>!\n";
        return;
    }
    unless (defined ($Genre = <FILE>))
    {
        close(FILE);
        warn "$Self: can't find field <Genre> in manifest file <$File>!\n";
        return;
    }

    $Artist =~ s!_+! !g;
    $Artist =~ s!^\s+!!;
    $Artist =~ s!\s+$!!;
    $Artist =~ s!\s+! !g;

    $Album =~ s!_+! !g;
    $Album =~ s!^\s+!!;
    $Album =~ s!\s+$!!;
    $Album =~ s!\s+! !g;

    $Year =~ s!_+! !g;
    $Year =~ s!^\s+!!;
    $Year =~ s!\s+$!!;
    $Year =~ s!\s+! !g;

    $Genre =~ s!_+! !g;
    $Genre =~ s!^\s+!!;
    $Genre =~ s!\s+$!!;
    $Genre =~ s!\s+! !g;

    $Count = 0;

    while (defined($Title = <FILE>) and @List)
    {
        $Title =~ s!_+! !g;
        $Title =~ s!^\s+!!;
        $Title =~ s!^\d+[\:\.\s-]*!!;
        $Title =~ s!\s+$!!;
        $Title =~ s!\s*\[[^\[\]]*\]$!!;
        $Title =~ s!\s*\.+[^\.\s\/\\]*$!!;
        $Title =~ s!\s+! !g;

        $Track = sprintf("%02d", ++$Count);

        $Filename = $WAV . '/' . shift(@List);

        $Outname = $Title;
        $Outname =~ s![\/\\\:\*\?\"\<\>\|\s]+!_!g;
        $Outname =~ s!_+(?:-+_+)+!_!g;
        $Outname =~ s!__+!_!g;
        $Outname =~ s!^_+!!;
        $Outname =~ s!_+$!!;
        $Newname = "${WAV}/${Track}_${Outname}.wav";
        $Outname = "${MP3}/${Track}_${Outname}.mp3";

        $CMD = 'lame.exe';
        $CMD .= qq( -b 320 -m s -h --add-id3v2);
        $CMD .= qq( --tt "$Title");
        $CMD .= qq( --ta "$Artist");
        $CMD .= qq( --tl "$Album");
        $CMD .= qq( --ty "$Year");
        $CMD .= qq( --tc "");
        $CMD .= qq( --tn "$Track");
        $CMD .= qq( --tg "$Genre");
        $CMD .= qq( "$Filename");
        $CMD .= qq( "$Outname");

        print "\n$CMD\n\n";
        system($CMD);

        if ($Filename ne $Newname)
        {
            unless (rename($Filename,$Newname))
            {
                warn "$Self: can't rename <$Filename> to <$Newname>: $!\n";
            }
        }
    }

    unless (close(FILE))
    {
        warn "$Self: can't close manifest file <$File>: $!\n";
    }
}

sub Normalize
{
    my $dir = $_[0] || '';
    my $drv = '';

    if ($dir =~ s!^([a-zA-Z]:)!!) { $drv = $1; }
    $dir = "/$dir/";
    $dir =~ s!\\!/!g;
    $dir =~ s!//+!/!g;
    while ($dir =~ s!/(?:\./)+!/!g) {};
    while ($dir =~ s,/(?!\.\./)[^/]+/\.\./,/,g) {};
    $dir =~ s!^/(?:\.\./)+!/!g;
    $dir =~ s!^/!!;
    $dir =~ s!/$!!;

    return wantarray ? ($drv,$dir) : "$drv/$dir";
}

sub MakeDir
{
    my($drv,$dir) = Normalize($_[0]);
    my(@dir);
    local($!);

    return '' if (-d "$drv/$dir");
    @dir = split(/\//, $dir);
    $dir = $drv;
    while (@dir)
    {
        $dir .= '/' . shift(@dir);
        unless (-d $dir)
        {
            unless (mkdir($dir,0777))
            {
                return "can't mkdir <$dir>: $!";
            }
        }
    }
    return '';
}

__END__
:endofperl
